"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs"));
const path_1 = require("path");
const typescript_1 = require("typescript");
const commandOptions_1 = tslib_1.__importStar(require("./commandOptions"));
const core_1 = tslib_1.__importStar(require("./core"));
const config_1 = tslib_1.__importStar(require("./core/config"));
function readConfig(options) {
    var _a;
    let pc = {};
    const configFile = (_a = options.configFile) !== null && _a !== void 0 ? _a : commandOptions_1.defaultConfigFile;
    try {
        pc = loadJSON(configFile);
        pc.configFile = configFile;
    }
    catch (err) {
        if (options.configFile != null) {
            console.error('Error to load config file from ' + options.configFile);
        }
    }
    if (pc.input == null) {
        pc.input = {
            files: [],
            urls: [],
            stdin: false,
        };
    }
    if (options.files.length > 0) {
        pc.input.files = options.files;
    }
    else if (pc.input.files == null) {
        pc.input.files = [];
    }
    if (options.urls.length > 0) {
        pc.input.urls = options.urls;
    }
    else if (pc.input.urls == null) {
        pc.input.urls = [];
    }
    pc.input.stdin = options.isReadFromStdin();
    if (options.out != null) {
        pc.outputFile = options.out;
    }
    if (options.target != null) {
        pc.target = convertToScriptTarget(options.target);
    }
    pc.outputAST = !!options.outputAST;
    return pc;
}
function loadJSON(file) {
    const content = fs.readFileSync(file, 'utf-8');
    return JSON.parse(content);
}
function convertToScriptTarget(target) {
    switch (target.trim().toLowerCase()) {
        case 'es3':
            return typescript_1.ScriptTarget.ES3;
        case 'es5':
            return typescript_1.ScriptTarget.ES5;
        case 'es2015':
            return typescript_1.ScriptTarget.ES2015;
        case 'es2016':
            return typescript_1.ScriptTarget.ES2016;
        case 'es2017':
            return typescript_1.ScriptTarget.ES2017;
        case 'es2018':
            return typescript_1.ScriptTarget.ES2018;
        case 'es2019':
            return typescript_1.ScriptTarget.ES2019;
        case 'es2020':
            return typescript_1.ScriptTarget.ES2020;
        case 'esnext':
            return typescript_1.ScriptTarget.ESNext;
        default:
            return typescript_1.ScriptTarget.Latest;
    }
}
async function loadContents() {
    let contents = [];
    const ps = [];
    if (config_1.default.input.stdin) {
        ps.push(core_1.readSchemaFromStdin().then((s) => {
            contents.push(s);
        }));
    }
    for (const pattern of config_1.default.input.files) {
        ps.push(core_1.readSchemasFromFile(pattern).then((ss) => {
            contents = contents.concat(ss);
        }));
    }
    for (const url of config_1.default.input.urls) {
        ps.push(core_1.readSchemaFromUrl(url).then((s) => {
            contents.push(s);
        }));
    }
    await Promise.all(ps);
    return contents;
}
async function exec() {
    const command = commandOptions_1.initialize(process.argv);
    const pc = readConfig(commandOptions_1.default);
    config_1.setConfig(pc);
    if (commandOptions_1.default.info) {
        const version = command.opts().version;
        await config_1.showConfig(version, config_1.default);
        return;
    }
    const contents = await loadContents();
    const result = await core_1.default({
        contents,
    });
    if (commandOptions_1.default.out) {
        fs.mkdirSync(path_1.dirname(commandOptions_1.default.out), { recursive: true });
        fs.writeFileSync(commandOptions_1.default.out, result, { encoding: 'utf-8' });
    }
    else {
        console.log(result);
    }
}
exec().catch((err) => {
    var _a;
    console.error((_a = err.stack) !== null && _a !== void 0 ? _a : err);
});
