"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globFiles = exports.readUrl = exports.readStream = void 0;
const tslib_1 = require("tslib");
const url_1 = require("url");
const glob_1 = tslib_1.__importDefault(require("glob"));
const https_proxy_agent_1 = tslib_1.__importDefault(require("https-proxy-agent"));
function readStream(stream, encoding = 'utf8') {
    stream.setEncoding(encoding);
    return new Promise((resolve, reject) => {
        let data = '';
        stream.on('data', (chunk) => (data += chunk));
        stream.on('end', () => resolve(data));
        stream.on('error', (error) => reject(error));
    });
}
exports.readStream = readStream;
async function readUrl(url) {
    const init = buildProxyOptions(url);
    const res = await fetch(url, init);
    const data = await res.text();
    if (!res.ok) {
        throw new Error(`Error on fetch from url(${url}): ${res.status}, ${data}`);
    }
    return data;
}
exports.readUrl = readUrl;
function noProxy(url) {
    if (process.env.NO_PROXY) {
        for (const domain of process.env.NO_PROXY.split(/[, ]+/)) {
            if (url.hostname.endsWith(domain)) {
                return true;
            }
        }
    }
    return false;
}
function buildProxyOptions(url) {
    const parsedUrl = new url_1.URL(url);
    let proxyUrl;
    if (!noProxy(parsedUrl)) {
        if (parsedUrl.protocol === 'http:' && process.env.HTTP_PROXY) {
            proxyUrl = new url_1.URL(process.env.HTTP_PROXY);
        }
        else if (parsedUrl.protocol === 'https:' && process.env.HTTPS_PROXY) {
            proxyUrl = new url_1.URL(process.env.HTTPS_PROXY);
        }
    }
    if (proxyUrl) {
        const agentOptions = {};
        agentOptions.protocol = proxyUrl.protocol;
        agentOptions.host = proxyUrl.hostname;
        agentOptions.port = proxyUrl.port;
        if (proxyUrl.username) {
            agentOptions.auth = proxyUrl.username + ':' + proxyUrl.password;
        }
        return { agent: https_proxy_agent_1.default(agentOptions) };
    }
    return undefined;
}
function globFiles(pattern, options) {
    return new Promise((resolve, reject) => {
        glob_1.default(pattern, options !== null && options !== void 0 ? options : {}, (err, matches) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(matches);
            }
        });
    });
}
exports.globFiles = globFiles;
